// FancyZoom.js - v1.1 - http://www.fancyzoom.com
//
// Copyright (c) 2008 Cabel Sasser / Panic Inc
// All rights reserved.
//
//     Requires: FancyZoomHTML.js
// Instructions: Include JS files in page, call setupZoom() in onLoad. That's it!
//               Any <a href> links to images will be updated to zoom inline.
//               Add rel="nozoom" to your <a href> to disable zooming for an image.
//
// Redistribution and use of this effect in source form, with or without modification,
// are permitted provided that the following conditions are met:
//
// * USE OF SOURCE ON COMMERCIAL (FOR-PROFIT) WEBSITE REQUIRES ONE-TIME LICENSE FEE PER DOMAIN.
//   Reasonably priced! Visit www.fancyzoom.com for licensing instructions. Thanks!
//
// * Non-commercial (personal) website use is permitted without license/payment!
//
// * Redistribution of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
//
// * Redistribution of source code and derived works cannot be sold without specific
//   written prior permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Drupal / JQuery version by Scott Lahteine www.thinkyhead.com
// Initialization moved to FancyZoomSettings.php.js
//

// Init. Do not add anything below this line, unless it's something awesome.

var browserIsIE = $.browser.msie,
	myWidth = 0, myHeight = 0, myScroll = 0, myScrollWidth = 0, myScrollHeight = 0,
	zoomOpen = false, preloadFrame = 1, preloadActive = false, imgPreload = $("<img/>"),
	preloadAnimTimer = 0,
	zoomActive = [], zoomTimer  = [],
	zoomOrigW  = [], zoomOrigH  = [], zoomOrigX  = [], zoomOrigY  = [],
	zoomBoxID		= "#ZoomBox",
	zoomShadowID	= "#ShadowBox",
	zoomImgID		= "#ZoomImage",
	zoomCloseID		= "#ZoomClose",
	zoomCaption		= "#ZoomCaption",
	zoomCaptionDiv	= "#ZoomCapDiv"
	;

$(function(){setupZoom()});

// setupZoom: Setup The Page! Called when the document is ready.

function setupZoom() {
	prepZooms();
	insertZoomHTML();
	zoomdiv = $(zoomBoxID);
	zoomimg = $(zoomImgID);
}

// Zoom: Inject Javascript functions into hrefs pointing to images, one by one!
// Skip any href that contains a rel="nozoom" tag.
// This is done at page load time via an onLoad() handler.

function prepZooms() {
	$("a").filter(
		function(i) { var h = $(this).attr("href"); return (h && h.search(/(.*)\.(jpg|jpeg|gif|png|bmp|tif|tiff)/gi) !== -1 && $(this).attr("rel") !== "nozoom"); }
	)
	.click(function (e) { return zoomClick(this, e); })
	.one("mouseover", function (e) { zoomPreload(this); });
}

// Zoom: Load an image into an image object. When done loading, function sets preloadActive to false,
// so other bits know that they can proceed with the zoom.
// Preloaded image is stored in imgPreload and swapped out in the zoom function.

function zoomPreload(from) {

	var href = $(from).attr("href");
	var isrc = imgPreload.attr("src");

	// Only preload if we have to, i.e. the image isn't this image already
	if (!isrc || isrc.indexOf(href.substr(href.lastIndexOf("/"))) === -1) {
		preloadActive = true;

		// Set a function to fire when the preload is complete, setting flags along the way.		
		imgPreload = $("<img/>")
			.load(function() { preloadActive = false; })
			.attr("src", href);
	}
}

// Zoom: Start the preloading animation cycle.

function preloadAnimStart() {
	$("#ZoomSpin").css({left:(myWidth/2)+"px",top:((myHeight/2)+myScroll)+"px",visibility:"visible"});
	preloadFrame = 1;
	$("#SpinImage").attr("src", zoomImagesURI+"zoom-spin-"+preloadFrame+".png");
	preloadAnimTimer = setInterval("preloadAnim()", 100);
}

// Zoom: Display and ANIMATE the jibber-jabber widget. Once preloadActive is false, bail and zoom it up!

function preloadAnim(from) {
	if (preloadActive) {
		$("#SpinImage").attr("src", zoomImagesURI+"zoom-spin-"+preloadFrame+".png");
		preloadFrame = (preloadFrame % 12) + 1;
	} else {
		$("#ZoomSpin").css("visibility", "hidden");
		clearInterval(preloadAnimTimer);
		preloadAnimTimer = 0;
		zoomIn(preloadFrom);
	}
}

// ZOOM CLICK: We got a click! Should we do the zoom? Or wait for the preload to complete?
// todo?: Double check that imgPreload src = clicked src

function zoomClick(from, evt) {

	var shift = evt.shiftKey;

	// Check for Command / Alt key. If pressed, pass them through -- don't zoom!
	if (! evt && window.event && (window.event.metaKey || window.event.altKey)) {
		return true;
	} else if (evt && (evt.metaKey|| evt.altKey)) {
		return true;
	}

	// Get Window Size

	myWidth = $(window).width();
	myHeight = $(window).height();
	myScroll = $(window).scrollTop();

	// Get Document size

	myScrollWidth = $(document).width();
	myScrollHeight = $(document).height();

	// If preloading still, wait, and display the spinner.
	if (preloadActive) {
		// But only display the spinner if it's not already being displayed!
		if (preloadAnimTimer === 0) {
			preloadFrom = from;
			preloadAnimStart();
		}
	} else {
		// Otherwise, we're loaded: do the zoom!
		zoomIn(from, shift);
	}

	return false;

}

// Zoom: Move an element in to endH endW, using zoomHost as a starting point.
// "from" is an object reference to the href that spawned the zoom.

function zoomIn(from, shift) {

	zoomimg.attr("src", $(from).attr("href"));

	// Determine the zoom settings from where we came from, the element in the <a>.
	// If there's no element in the <a>, or we can't get the width, make stuff up

	var kid = $(from).children()[0];

	if (kid.width) {
		startW = kid.width;
		startH = kid.height;
		offs = $(kid).offset();
	} else {
		startW = 50;
		startH = 12;
		offs = $(from).offset();
	}

	hostX = offs.left;
	hostY = offs.top;

	// Make up for a scrolled containing div.

	if ($("#scroller").length) {
		hostX -= $("#scroller")[0].scrollLeft;
	}

	// Determine the target zoom settings from the preloaded image object

	endW = $(imgPreload)[0].width;
	endH = $(imgPreload)[0].height;

	// Start! But only if we're not zooming already!

	if (zoomActive[zoomImgID] !== true) {

		// Hide shadow and close box
		hideZoomExtras();

		// Setup the CAPTION, if existing. Hide it first, set the text.

		if (includeCaption) {
			$(zoomCaptionDiv).css("visibility", "hidden");
			if ($(from).attr("title")) {
				// Yes, there's a caption, set it up
				$(zoomCaption).html($(from).attr("title"));
			} else {
				$(zoomCaption).html("");
			}
		}

		// Store original position in an array for future zoomOut.

		zoomOrigW[zoomImgID] = startW;
		zoomOrigH[zoomImgID] = startH;
		zoomOrigX[zoomImgID] = hostX;
		zoomOrigY[zoomImgID] = hostY;

		// Now set the starting dimensions

		zoomimg.width(startW).height(startH);
		zoomdiv.css({left:hostX+"px", top:hostY+"px"});

		// Show the zooming image container, make it invisible

		if (includeFade === 1) {
			setOpacity(0, zoomBoxID);
		}
		zoomdiv.css("visibility", "visible");

		// If it's too big to fit in the window, shrink the width and height to fit (with ratio).

		sizeRatio = endW / endH;
		if (endW > myWidth - minBorder) {
			endW = myWidth - minBorder;
			endH = endW / sizeRatio;
		}
		if (endH > myHeight - minBorder) {
			endH = myHeight - minBorder;
			endW = endH * sizeRatio;
		}

		zoomChangeX = (((myWidth - endW) / 2) - hostX);
		zoomChangeY = (((myHeight - endH) / 2) - hostY) + myScroll;
		zoomChangeW = (endW - startW);
		zoomChangeH = (endH - startH);

		// Shift key?

		tempSteps = zoomSteps * (shift ? 7 : 1);

		// Setup Zoom

		zoomCurrent = 0;

		// Setup Fade with Zoom, If Requested

		fadeCurrent = 0;
		fadeAmount = includeFade ? -100 / tempSteps : 0;

		// The JQuery way to do this:
//		$(zoomBoxID).animate({left:parseInt(hostX+zoomChangeX,10)+"px", top:parseInt(hostY+zoomChangeY,10)+"px"});
//		$(zoomImgID).animate({width: endW+"px", height: endH+"px"}, "normal", "", function(){zoomDoneIn(zoomBoxID)});

		// Do It!

		zoomTimer[zoomImgID] = setInterval("zoomElement('"+zoomBoxID+"', '"+zoomImgID+"', "+zoomCurrent+", "+startW+", "+zoomChangeW+", "+startH+", "+zoomChangeH+", "+hostX+", "+zoomChangeX+", "+hostY+", "+zoomChangeY+", "+tempSteps+", "+includeFade+", "+fadeAmount+", 'zoomDoneIn(zoomBoxID)')", zoomTime);
		zoomActive[zoomImgID] = true;
	}
}

function hideZoomExtras() {
	if ($(zoomShadowID).length) {
		$(zoomShadowID).css("visibility", "hidden");
	} else if (!browserIsIE) {

		// Wipe timer if shadow is fading in still
		if (fadeActive[zoomImgID]) {
			clearInterval(fadeTimer[zoomImgID]);
			fadeActive[zoomImgID] = false;
			fadeTimer[zoomImgID] = false;
		}

		$(zoomImgID).css("-webkit-box-shadow", shadowSettings + "0.0)");
	}

	$(zoomCloseID).css("visibility", "hidden");

	if (includeCaption) {
		$(zoomCaptionDiv).css("visibility", "hidden");
	}
}

// Zoom it back out.

function zoomOut(from, evt) {

	// Get shift key status.
	// IE events don't seem to get passed through the function, so grab it from the window.

	tempSteps = zoomSteps * (evt.shiftKey ? 7 : 1);

	// Check to see if something is happening/open

	if (zoomActive[zoomImgID] !== true) {

		// First, get rid of the shadow, close box, and caption
		hideZoomExtras();

/*		if (includeCaption && $(zoomCaption).html() !== "") {
			fadeElementSetup(zoomCaptionDiv, 100, 0, 5, 1);
		}
*/
		// Now, figure out where we came from, to get back there

		startX = parseInt(zoomdiv.css("left"), 10);
		startY = parseInt(zoomdiv.css("top"), 10);
		startW = zoomimg.width();
		startH = zoomimg.height();
		zoomChangeX = zoomOrigX[zoomImgID] - startX;
		zoomChangeY = zoomOrigY[zoomImgID] - startY;
		zoomChangeW = zoomOrigW[zoomImgID] - startW;
		zoomChangeH = zoomOrigH[zoomImgID] - startH;

		// Setup Zoom

		zoomCurrent = 0;

		// Setup Fade with Zoom, If Requested

		if (includeFade === 1) {
			fadeCurrent = 0;
			fadeAmount = (100 - 0) / tempSteps;
		} else {
			fadeAmount = 0;
		}

		// Do It!

		zoomTimer[zoomImgID] = setInterval("zoomElement('"+zoomBoxID+"', '"+zoomImgID+"', "+zoomCurrent+", "+startW+", "+zoomChangeW+", "+startH+", "+zoomChangeH+", "+startX+", "+zoomChangeX+", "+startY+", "+zoomChangeY+", "+tempSteps+", "+includeFade+", "+fadeAmount+", 'zoomDone(zoomBoxID, zoomImgID)')", zoomTime);
		zoomActive[zoomImgID] = true;
	}
}

// Finished Zooming In

function zoomDoneIn(zoomID, theID) {

	// Note that it's open

	zoomOpen = true;
	zoomdiv = $(zoomID);

	// Position the table shadow behind the zoomed in image, and display it

	shbox = $(zoomShadowID);
	if (shbox.length) {

		setOpacity(0, zoomShadowID);

		shadowLeft = parseInt(zoomdiv.css("left"), 10) - 13;
		shadowTop = parseInt(zoomdiv.css("top"), 10) - 8;
		shadowWidth = zoomdiv[0].offsetWidth + 26;
		shadowHeight = zoomdiv[0].offsetHeight + 26;

		shbox.width(shadowWidth).height(shadowHeight).css({left:shadowLeft+"px", top:shadowTop+"px", visibility:"visible"});
		fadeElementSetup(zoomShadowID, 0, 100, 5);

	} else if (!browserIsIE) {
		// Or, do a fade of the modern shadow
		fadeElementSetup(zoomImgID, 0, .8, 5, 0, "shadow");
	}

	// Position and display the CAPTION, if existing

	if (includeCaption && $(zoomCaption).html() !== "") {
		zoomcapd = $(zoomCaptionDiv);
//		if (!browserIsIE) setOpacity(0, zoomCaptionDiv);
		zoomcapd.css({
			top: parseInt(zoomdiv.css("top"), 10) + (zoomdiv[0].offsetHeight + 15) + "px",
			left: (myWidth / 2) - (zoomcapd[0].offsetWidth / 2) + "px",
			visibility: "visible"});
//		if (!browserIsIE) fadeElementSetup(zoomCaptionDiv, 0, 100, 5);
	}

	// Display Close Box (fade it if it's not IE)
	if (browserIsIE) {
		$(zoomCloseID).css("visibility", "visible");
	} else {
		setOpacity(0, zoomCloseID);
		$(zoomCloseID).css("visibility", "visible");	// .fadeTo(500, 1);
		fadeElementSetup(zoomCloseID, 0, 100, 5);
	}

	// Get keypresses
	$(document).keypress(function(e){ if (e.keyCode === 27) {zoomOut(this, e)} });

}

// Finished Zooming Out

function zoomDone(zoomID, theID) {

	// No longer open

	zoomOpen = false;

	// Clear stuff out, clean up

	zoomOrigH[theID] = zoomOrigW[theID] = "";
	$(zoomID).css("visibility", "hidden");
	zoomActive[theID] === false;

	// Stop getting keypresses

	$(document).unbind("keypress");

}

// Actually zoom the element

function zoomElement(zoomID, theID, zoomCurrent, zoomStartW, zoomChangeW, zoomStartH, zoomChangeH, zoomStartX, zoomChangeX, zoomStartY, zoomChangeY, zoomSteps, includeFade, fadeAmount, execWhenDone) {

	// console.log("Zooming Step #"+zoomCurrent+ " of "+zoomSteps+" (zoom " + zoomStartW + "/" + zoomChangeW + ") (zoom " + zoomStartH + "/" + zoomChangeH + ")  (zoom " + zoomStartX + "/" + zoomChangeX + ")  (zoom " + zoomStartY + "/" + zoomChangeY + ") Fade: "+fadeAmount);

	// Test if we're done, or if we continue

	if (zoomCurrent === (zoomSteps + 1)) {
		zoomActive[theID] = false;
		clearInterval(zoomTimer[theID]);

		if (execWhenDone !== "") {
			eval(execWhenDone);
		}
	} else {

		// Do the Fade!

		if (includeFade === 1) {
			fade = zoomCurrent * fadeAmount;
			setOpacity((fadeAmount < 0) ? Math.abs(fade) : 100 - fade, zoomID);
		}

		// Calculate this step's difference, and move it!

		moveW = cubicInOut(zoomCurrent, zoomStartW, zoomChangeW, zoomSteps);
		moveH = cubicInOut(zoomCurrent, zoomStartH, zoomChangeH, zoomSteps);
		moveX = cubicInOut(zoomCurrent, zoomStartX, zoomChangeX, zoomSteps);
		moveY = cubicInOut(zoomCurrent, zoomStartY, zoomChangeY, zoomSteps);

		$(zoomID).css({left: moveX+"px", top: moveY+"px"});
		zoomimg.width(moveW).height(moveH);

		zoomCurrent++;

		clearInterval(zoomTimer[theID]);
		zoomTimer[theID] = setInterval("zoomElement('"+zoomID+"', '"+theID+"', "+zoomCurrent+", "+zoomStartW+", "+zoomChangeW+", "+zoomStartH+", "+zoomChangeH+", "+zoomStartX+", "+zoomChangeX+", "+zoomStartY+", "+zoomChangeY+", "+zoomSteps+", "+includeFade+", "+fadeAmount+", '"+execWhenDone+"')", zoomTime);
	}
}

// Fade: Initialize the fade function

var fadeActive = [], fadeQueue = [], fadeTimer = [], fadeClose = [], fadeMode = [];

function fadeElementSetup(theID, fdStart, fdEnd, fdSteps, fdClose, fdMode) {

	// alert("Fading: "+theID+" Steps: "+fdSteps+" Mode: "+fdMode);

	if (fadeActive[theID] === true) {
		// Already animating, queue up this command
		fadeQueue[theID] = [theID, fdStart, fdEnd, fdSteps];
	} else {
		fadeSteps = fdSteps;
		fadeCurrent = 0;
		fadeAmount = (fdStart - fdEnd) / fadeSteps;
		fadeTimer[theID] = setInterval("fadeElement('"+theID+"', '"+fadeCurrent+"', '"+fadeAmount+"', '"+fadeSteps+"')", 15);
		fadeActive[theID] = true;
		fadeMode[theID] = fdMode;

		fadeClose[theID] = (fdClose === 1);
	}
}

// Fade: Do the fade. This function will call itself, modifying the parameters, so
// many instances can run concurrently. Can fade using opacity, or fade using a box-shadow.

function fadeElement(theID, fadeCurrent, fadeAmount, fadeSteps) {

	if (fadeCurrent === fadeSteps) {

		// We're done, so clear.

		clearInterval(fadeTimer[theID]);
		fadeActive[theID] = false;
		fadeTimer[theID] = false;

		// Should we close it once the fade is complete?

		if (fadeClose[theID] === true) {
			$(theID).css("visibility", "hidden");
		}

		// Hang on.. did a command queue while we were working? If so, make it happen now

		if (fadeQueue[theID] && fadeQueue[theID] !== false) {
			fadeElementSetup(fadeQueue[theID][0], fadeQueue[theID][1], fadeQueue[theID][2], fadeQueue[theID][3]);
			fadeQueue[theID] = false;
		}
	} else {

		fadeCurrent++;

		fade = fadeCurrent * fadeAmount;
		fade = (fadeAmount < 0) ? Math.abs(fade) : 100 - fade;

		// Now actually do the fade adjustment.
		if (fadeMode[theID] === "shadow") {
			// Do a special fade on the webkit-box-shadow of the object
			$(theID).css("-webkit-box-shadow", shadowSettings + fade + ")");
		} else {
			// Set the opacity depending on if we're adding or subtracting (pos or neg)
			setOpacity(fade, theID);
		}

		// Keep going, and send myself the updated variables
		clearInterval(fadeTimer[theID]);
		fadeTimer[theID] = setInterval("fadeElement('"+theID+"', '"+fadeCurrent+"', '"+fadeAmount+"', '"+fadeSteps+"')", 15);
	}
}

////////////////////////////
//
// UTILITY functions
//

// Utility: Set the opacity, compatible with a number of browsers. Value from 0 to 100.

function setOpacity(opacity, theID) {

	// If it's 100, set it to 99 for Firefox (SRL: Mozilla?).

	if ($.browser.mozilla && opacity === 100) {
		opacity = 99.9999; // This is majorly awkward
	}

	// Multi-browser opacity setting:
	//						IE/Win						Safari 1.2, Firefox+Mozilla
	$(theID).css({filter:"alpha(opacity="+opacity+")", opacity:(opacity/100)});
}

// Utility: Math functions for animation calucations - From http://www.robertpenner.com/easing/
//
// t = time, b = begin, c = change, d = duration
// time = current frame, begin is fixed, change is basically finish - begin, duration is fixed (frames),

function cubicInOut(t, b, c, d)
{
	return ((t/=d/2) < 1) ? c/2*t*t*t + b : c/2*((t-=2)*t*t + 2) + b;
}
